"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.WindowManager = void 0;
const electron_1 = require("electron");
const fs_1 = require("fs");
const os_1 = require("os");
const path_1 = require("path");
const recursive_copy_1 = __importDefault(require("recursive-copy"));
const startServer_1 = require("./startServer");
const userData_1 = require("./userData");
let projectPath = userData_1.userData.getProjectPath();
class WindowManager {
    static count = 0;
    win;
    _serverProc;
    constructor() {
        WindowManager.count++;
        this.win = new electron_1.BrowserWindow({
            width: 1200 + Math.floor(Math.random() * 200),
            height: 800 + Math.floor(Math.random() * 200),
            center: false,
            autoHideMenuBar: true,
            show: true,
            webPreferences: {
                nodeIntegration: false,
                contextIsolation: true,
                enableRemoteModule: false,
                preload: (0, path_1.join)(__dirname, "preload.js")
            }
        });
        this.createMenu();
        if (process.platform === "linux") {
            const icon = electron_1.nativeImage.createFromPath((0, path_1.join)(__dirname, "../../linux-assets/icon.png"));
            this.win.setIcon(icon);
        }
        const ipcMainListener = async (event, arg) => {
            if (event.sender !== this.win.webContents) {
                return;
            }
            console.log("ipcMain.on:");
            console.log(arg);
            const method = arg.method;
            const body = arg.body || {};
            switch (method) {
                case "ask-close-window": {
                    const choice = await electron_1.dialog.showMessageBox(this.win, {
                        type: 'question',
                        buttons: ['Leave', 'Stay'],
                        title: 'Do you want to leave?',
                        message: 'Changes you made may not be saved.',
                        defaultId: 0,
                        cancelId: 1
                    });
                    console.log(choice);
                    const leave = (choice.response === 0);
                    if (leave) {
                        console.log("close window");
                        this.win.destroy();
                    }
                    break;
                }
                case "new-window": {
                    new WindowManager();
                    break;
                }
                case "clear-list": {
                    this.clearList();
                    break;
                }
                case "show-new-project-page": {
                    this.loadNewProjectPage();
                    break;
                }
                case "open-project": {
                    let dir = body.project;
                    if (dir === undefined) {
                        const result = await electron_1.dialog.showOpenDialog(this.win, {
                            message: "Select Folder",
                            properties: ["openDirectory", "createDirectory", "promptToCreate"],
                            defaultPath: projectPath
                        });
                        dir = result ? result.filePaths[0] : undefined;
                    }
                    this.openProject(dir);
                    break;
                }
                case "close-project": {
                    userData_1.userData.deleteProjectPath();
                    this.loadHomePage();
                    this.killServerProcess();
                    break;
                }
                case "create-project": {
                    try {
                        const result = await electron_1.dialog.showOpenDialog(this.win, {
                            message: "Select Project Path",
                            properties: ["openDirectory", "createDirectory", "promptToCreate"],
                            defaultPath: projectPath || (0, os_1.homedir)()
                        });
                        if (!result.canceled) {
                            this.win.loadFile("src/html/loading.html");
                            const dir = result.filePaths[0];
                            (0, fs_1.mkdirSync)(dir, { recursive: true });
                            const src = (0, path_1.join)(electron_1.app.getAppPath(), "starter-templates", body.repo);
                            await (0, recursive_copy_1.default)(src, dir, {
                                dot: true,
                                overwrite: false,
                                results: false,
                            });
                            this.openProject(dir);
                        }
                    }
                    catch (e) {
                        console.log(e);
                        electron_1.dialog.showErrorBox("Error", e.message);
                        this.loadHomePage();
                    }
                    break;
                }
                case "recent-projects": {
                    const projects = userData_1.userData.getRecentProjects();
                    event.returnValue = projects;
                    break;
                }
                case "open-dev-tools": {
                    this.win.webContents.openDevTools({
                        mode: "bottom"
                    });
                    break;
                }
            }
        };
        electron_1.ipcMain.on("electron-phasereditor2d", ipcMainListener);
        this.win.once("closed", () => {
            electron_1.ipcMain.removeListener("electron-phasereditor2d", ipcMainListener);
            WindowManager.count--;
            this.killServerProcess();
        });
        if (WindowManager.count === 1 && projectPath && (0, fs_1.existsSync)(projectPath) && (0, fs_1.statSync)(projectPath).isDirectory()) {
            this.openProject(projectPath);
        }
        else {
            this.loadHomePage();
        }
    }
    killServerProcess() {
        if (this._serverProc) {
            console.log(`Kill server proc ${this._serverProc.pid}`);
            this._serverProc.kill("SIGKILL");
            this._serverProc = undefined;
        }
        else {
            console.log("No server process to kill.");
        }
    }
    clearList() {
        userData_1.userData.clearRecentProjects();
    }
    loadHomePage() {
        this.win.loadFile("src/html/start.html");
    }
    loadNewProjectPage() {
        this.win.loadFile("src/html/newProject.html");
    }
    async openProject(project) {
        if (!project) {
            return;
        }
        if (!(0, fs_1.existsSync)(project) || !(0, fs_1.statSync)(project).isDirectory()) {
            electron_1.dialog.showMessageBox(this.win, {
                type: "question",
                buttons: ["Close"],
                title: "File not found",
                message: `File "${project}" does not exist or is not a directory.`,
                defaultId: 0,
                cancelId: 1
            });
            userData_1.userData.deleteRecentProject(project);
            userData_1.userData.deleteProjectPath();
            this.loadHomePage();
            return;
        }
        const { port, proc } = await (0, startServer_1.startServer)(project);
        this._serverProc = proc;
        const url = `http://127.0.0.1:${port}/editor/`;
        setTimeout(() => this.win.loadURL(url), 500);
        projectPath = project;
        userData_1.userData.setProjectPath(projectPath);
        userData_1.userData.incrementRecentProject(project);
        return true;
    }
    createMenu() {
        const isMac = process.platform === 'darwin';
        const template = [
            // { role: 'appMenu' }
            ...(isMac ? [{
                    label: "Phaser Editor 2D",
                    submenu: [
                        { role: 'about' },
                        { type: 'separator' },
                        { role: 'services' },
                        { type: 'separator' },
                        { role: 'hide' },
                        { role: 'hideothers' },
                        { role: 'unhide' },
                        { type: 'separator' },
                        { role: 'quit' }
                    ]
                }] : []),
            // { role: 'fileMenu' }
            {
                label: 'File',
                submenu: [
                    isMac ? { role: 'close' } : { role: 'quit' }
                ]
            },
            // { role: 'editMenu'}
            {
                label: 'Edit',
                submenu: [
                    { role: 'undo' },
                    { role: 'redo' },
                    { type: 'separator' },
                    { role: 'cut' },
                    { role: 'copy' },
                    { role: 'paste' },
                    ...(isMac ? [
                        { role: 'pasteAndMatchStyle' },
                        { role: 'delete' },
                        { role: 'selectAll' },
                        { type: 'separator' },
                        {
                            label: 'Speech',
                            submenu: [
                                { role: 'startSpeaking' },
                                { role: 'stopSpeaking' }
                            ]
                        }
                    ] : [
                        { role: 'delete' },
                        { type: 'separator' },
                        { role: 'selectAll' }
                    ])
                ]
            },
            // { role: 'viewMenu' }
            {
                label: 'View',
                submenu: [
                    { role: 'reload' },
                    { role: 'forceReload' },
                    { role: 'toggleDevTools' },
                    { type: 'separator' },
                    { role: 'resetZoom' },
                    { role: 'zoomIn' },
                    { role: 'zoomOut' },
                    { type: 'separator' },
                    { role: 'togglefullscreen' }
                ]
            },
            // { role: 'windowMenu' }
            {
                label: 'Window',
                submenu: [
                    { role: 'minimize' },
                    { role: 'zoom' },
                    ...(isMac ? [
                        { type: 'separator' },
                        { role: 'front' },
                        { type: 'separator' },
                        { role: 'window' }
                    ] : [
                        { role: 'close' }
                    ])
                ]
            }
        ];
        const menu = electron_1.Menu.buildFromTemplate(template);
        electron_1.Menu.setApplicationMenu(menu);
    }
    exitApp() {
        process.exit();
    }
}
exports.WindowManager = WindowManager;
